"""
 HTTP Post Sample WebServer
"""
import datetime
import io
import re

from http.server import HTTPServer, SimpleHTTPRequestHandler

"""
 Sample WebServer class
"""
class HTTPPostSampleHandler(SimpleHTTPRequestHandler):

	"""
	 HTTP Get method
	"""
	def do_GET(self):
		print(" --- GET Request ---")
		body = b'<html><head><meta charset="utf-8"/><title>GET result</title></head><body>HTTP GET OK</body></html>'
		self.send_response(200)
		self.send_header('Content-type', 'text/html; charset=utf-8')
		self.send_header('Content-length', len(body))
		self.end_headers()
		self.wfile.write(body)
		print("  ===> OK")
		return

	"""
	 HTTP Post method
	"""
	def do_POST(self):
		print(" --- POST Request ---")
		content_len  = int(self.headers.get("content-length", 0))
		post_body = self.rfile.read(content_len)

		self.write_header_and_body(self.headers, post_body)

		html = self.create_post_result_html(self.headers, post_body)
		htmlEnc = html.encode('utf-8')

		self.send_response(200)
		self.send_header('Content-type', 'text/html; charset=utf-8')
		self.send_header('Content-length', len(htmlEnc))
		self.end_headers()

		self.wfile.write(htmlEnc)
		print("  ===> OK")
		return

	"""
	 Output file HTTP Post Header and Body information
	"""
	def write_header_and_body(self, headers, body):

		dt = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
		with open("post_header" + dt + ".txt", "w") as f:
			for header in headers:
				f.write(header + " : " + headers[header] + "\n")

		with open("post_body" + dt + ".txt", "wb") as f:
			f.write(body)

		return

	"""
	 Create HTTP page from HTTP Post Header and Body
	"""
	def create_post_result_html(self, headers, body):

		html = '<html><head><meta charset="utf-8"/><title>POST result</title></head><body>HTTP POST OK<br><br>POST Header:<br>'
		html += '<textarea id="header_id" name="header" cols="100" rows="15" wrap="off" readonly>'

		for header in headers:
			html += header + " : " + self.headers[header] + "\r\n"

		html += "</textarea>"

		encodeStr = body.decode(encoding='utf-8', errors='replace')

		html += "<br><br>POST Body :<br>"
		html += '<textarea id="body_id" name="body" cols="100" rows="20" wrap="off" readonly>'

		html += encodeStr + "</textarea></body></html>"
		return html

try:
	host = 'localhost'
	port = 80
	httpd = HTTPServer((host, port), HTTPPostSampleHandler)
	print('Boot Web Server(host = ' + host + ', port = ' + str(port) + ')')
	print('[Web Server abort press "Ctrl+C" Key]')
	httpd.serve_forever()

except KeyboardInterrupt:
	print('Ctrl+C key received, Shutdown Web Server')
	httpd.socket.close()
